from zope.interface import implements
from repoze.who.interfaces import IAuthenticator, IMetadataProvider
from repoze.who.plugins.friendlyform import FriendlyFormPlugin
from repoze.who.plugins.auth_tkt import AuthTktCookiePlugin
from repoze.who.plugins.testutil import make_middleware

class MingAuthenticatorPlugin(object):
    implements(IAuthenticator)

    def __init__(self, user_class):
        self.user_class = user_class

    # IAuthenticator
    def authenticate(self, environ, identity):
        if not ('login' in identity and 'password' in identity):
            return None

        user = self.user_class.query.get(user_name=identity.get('login', None))
        if user:
            if user.validate_password(identity.get('password', None)):
                return identity['login']

class MingUserMDPlugin(object):
    implements(IMetadataProvider)

    def __init__(self, user_class):
        self.user_class = user_class

    def add_metadata(self, environ, identity):
        identity['user'] = self.user_class.query.get(user_name=identity['repoze.who.userid'])

        if identity['user']:
            identity['groups'] = [g.group_name for g in identity['user'].groups]
            identity['permissions'] = [p.permission_name for p in identity['user'].permissions]
        else:
            identity['groups'] = []
            identity['permissions'] = []

        if 'repoze.what.credentials' not in environ:
            environ['repoze.what.credentials'] = {}

        environ['repoze.what.credentials']['groups'] = identity['groups']
        environ['repoze.what.credentials']['permissions'] = identity['permissions']
        environ['repoze.what.credentials']['repoze.what.userid'] = identity['repoze.who.userid']

def setup_ming_auth(app, skip_authentication, **auth_args):
    cookie_secret = auth_args.get('cookie_secret', 'secret')
    cookie_name = auth_args.get('cookie_name', 'authtkt')

    form_plugin = FriendlyFormPlugin(auth_args.get('login_url', '/login'),
                                     auth_args.get('login_handler', '/login_handler'),
                                     auth_args['post_login_url'],
                                     auth_args.get('logout_handler', '/logout_handler'),
                                     auth_args['post_logout_url'],
                                     login_counter_name=auth_args.get('login_counter_name'),
                                     rememberer_name='cookie')

    challengers = [('form', form_plugin)]

    auth = MingAuthenticatorPlugin(user_class=auth_args['user_class'])
    authenticators = [('mingauth', auth)]

    cookie = AuthTktCookiePlugin(cookie_secret, cookie_name)
    identifiers = [('cookie', cookie), ('form', form_plugin)]

    provider = MingUserMDPlugin(user_class=auth_args['user_class'])
    mdproviders = [('ming_user_md', provider)]

    from repoze.who.classifiers import default_request_classifier
    from repoze.who.classifiers import default_challenge_decider

    app = make_middleware(skip_authentication,
        app,
        identifiers=identifiers,
        authenticators=authenticators,
        challengers=challengers,
        mdproviders=mdproviders,
        classifier=default_request_classifier,
        challenge_decider=default_challenge_decider)

    return app



from ming import schema as s
from ming.orm import FieldProperty, RelationProperty
from ming.orm.property import ORMProperty, LazyProperty, OneToManyJoin
from pymongo.objectid import ObjectId

class SynonymProperty(ORMProperty):
    include_in_repr = True

    def __init__(self, getter, setter=None):
        super(ORMProperty, self).__init__()
        self.getter = getter
        self.setter = setter

    def __get__(self, instance, cls=None):
        if not instance:
            return self
        return self.getter(instance)

    def __set__(self, instance, value):
        if not self.setter:
            raise TypeError, 'read-only property'
        else:
            self.setter(instance, value)

    def repr(self, doc):
        return repr(self)

    def __repr__(self):
        return '<%s>' % (self.__class__.__name__,)

class ProgrammaticRelationProperty(RelationProperty):
    include_in_repr = False

    def __init__(self, related, getter, setter=None, relation_field=None):
        super(ProgrammaticRelationProperty, self).__init__(related)
        self.relation_field = relation_field
        self.getter = getter
        self.setter = setter

    def __get__(self, instance, cls=None):
        if not instance:
            return self
        return self.getter(instance)

    def __set__(self, instance, value):
        if not self.setter:
            raise TypeError, 'read-only property'
        else:
            self.setter(instance, value)

    @LazyProperty
    def join(self):
        return OneToManyJoin(self.mapper.mapped_class, self.related, self.relation_field)




from setuptools import setup, find_packages
import sys, os

version = '0.0.4'

here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.rst')).read()
except IOError:
    README = ''

setup(name='tgming',
      version=version,
      description="TurboGears2 Support for Ming MongoDB ORM",
      long_description=README,
      classifiers=[
        "Environment :: Web Environment",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: TurboGears"
      ], 
      keywords='turbogears2.extension',
      author='Alessandro Molina',
      author_email='alessandro.molina@axant.it',
      url='http://bitbucket.org/_amol_/turbogears-ming',
      license='MIT',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        "repoze.who-friendlyform >= 1.0.4",
        "repoze.who >= 1.0.19",
        "repoze.who-testutil >= 1.0.1",
        "ming"
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
